Qua12

Struct Qua12 

pub struct Qua12 {}
Defines a quadrilateral with 12 nodes (cubic edges)

qua12

Local IDs of nodes

 3----10-------6------2
 |               (1,1)|
 |       s ^          |
 7         |          9
 |         |          |
 |         +----> r   |
 |       (0,0)        |
11                    5
 |                    |
 |(-1,-1)             |
 0-----4-------8------1

Local IDs of edges

             2
   3----10-------6------2
   |                    |
   |                    |
   7                    9          p0 p1  p2 p3
   |                    |      e:0 [1, 0,  8, 4]
3  |                    | 1    e:1 [2, 1,  9, 5]
   |                    |      e:2 [3, 2, 10, 6]
  11                    5      e:3 [0, 3, 11, 7]
   |                    |
   |                    |
   0-----4-------8------1
             0

Notes

  • The reference coordinates range from -1 to +1 with the geometry centred @ 0
  • The order of edge nodes is such that the normals are outward
  • The order of edge nodes corresponds to super::Lin4 nodes

Implementations

Source

impl Qua12

pub const GEO_NDIM: usize = 2usize

pub const NNODE: usize = 12usize

pub const NEDGE: usize = 4usize

pub const NFACE: usize = 0usize

pub const EDGE_NNODE: usize = 4usize

pub const FACE_NNODE: usize = 0usize

pub const FACE_NEDGE: usize = 0usize

pub const TRIANGULATE_NTRIANGLE: usize = 18usize

pub const TRIANGULATE_EXTRA_NNODE: usize = 4usize

pub const EDGE_NODE_IDS: [[usize; 4]; 4]

pub const EDGE_NODE_IDS_INWARD: [[usize; 4]; 4]

pub const NODE_REFERENCE_COORDS: [[f64; 2]; 12]

pub const TRIANGULATE_TRIANGLES: [[usize; 3]; 18]

pub const TRIANGULATE_EXTRA_COORDS: [[f64; 2]; 4]

pub fn calc_interp(interp: &mut Vector, ksi: &[f64])

Computes the interpolation functions

Output
  • interp – interpolation function evaluated at ksi (nnode)
Input
  • ksi – reference coordinates with length ≥ geo_ndim
pub fn calc_deriv(deriv: &mut Matrix, ksi: &[f64])

Computes the derivatives of interpolation functions with respect to the reference coordinates

Output
  • deriv – derivatives of the interpolation function with respect to the reference coordinates ksi, evaluated at ksi (nnode,geo_ndim)
Input
  • ksi – reference coordinates with length ≥ geo_ndim